<!doctype html>
<html>
<head>
 <link rel="stylesheet" href="/static/style.css"/>
</head>
<body>
  <div class="banner">OAuthToolKit - Fake OAuth Server</div>
  {% block page %}{% endblock %}
</body>
</html>
