{% extends "dialog.html" %}
{% block title %}Authorize...{% endblock %}
{% block content %}
<div>
  <div>
    Application "{{grant.client.client_name}}" is requesting your consent...<br>
    Requested scopes: {{ grant.request.scope }}.
    <br><br>
  </div>

  <form action="" method="post">
    <label>
      <input type="checkbox" name="confirm">
      <span>Consent?</span>
    </label>
    <br>
    <button>Submit</button>
  </form>
</div>
{% endblock %}
